"""
Event definitions for AIKademiya platform
Implements Event-Driven Architecture with Kafka
"""

from pydantic import BaseModel
from datetime import datetime
from typing import Dict, Any, Optional
from enum import Enum


class EventType(str, Enum):
    # User Events
    USER_REGISTERED = "user.registered"
    USER_UPDATED = "user.updated"
    USER_SUBSCRIPTION_CHANGED = "user.subscription_changed"
    
    # Learning Events
    TRACK_GENERATION_REQUESTED = "learning.track_generation_requested"
    TRACK_GENERATION_COMPLETED = "learning.track_generation_completed"
    TRACK_GENERATION_FAILED = "learning.track_generation_failed"
    LESSON_STARTED = "learning.lesson_started"
    LESSON_COMPLETED = "learning.lesson_completed"
    MODULE_COMPLETED = "learning.module_completed"
    TRACK_COMPLETED = "learning.track_completed"
    
    # AI Agent Events
    AI_GENERATION_STARTED = "ai.generation_started"
    AI_GENERATION_COMPLETED = "ai.generation_completed"
    AI_GENERATION_FAILED = "ai.generation_failed"
    AI_FEEDBACK_RECEIVED = "ai.feedback_received"
    
    # Notification Events
    NOTIFICATION_REQUESTED = "notification.requested"
    NOTIFICATION_SENT = "notification.sent"
    NOTIFICATION_FAILED = "notification.failed"


class BaseEvent(BaseModel):
    """Base event class"""
    event_type: EventType
    event_id: str
    timestamp: datetime
    user_id: Optional[int] = None
    correlation_id: Optional[str] = None
    source_service: str
    data: Dict[str, Any]


class UserRegisteredEvent(BaseEvent):
    """Событие регистрации пользователя"""
    event_type: EventType = EventType.USER_REGISTERED


class TrackGenerationRequestedEvent(BaseEvent):
    """Событие запроса генерации трека"""
    event_type: EventType = EventType.TRACK_GENERATION_REQUESTED


class TrackGenerationCompletedEvent(BaseEvent):
    """Событие завершения генерации трека"""
    event_type: EventType = EventType.TRACK_GENERATION_COMPLETED


class LessonCompletedEvent(BaseEvent):
    """Событие завершения урока"""
    event_type: EventType = EventType.LESSON_COMPLETED


class NotificationRequestedEvent(BaseEvent):
    """Событие запроса уведомления"""
    event_type: EventType = EventType.NOTIFICATION_REQUESTED


import os
import redis.asyncio as redis
import logging


REDIS_URL = os.getenv("REDIS_URL", "redis://localhost:6379")
logger = logging.getLogger(__name__)

redis_client = redis.from_url(REDIS_URL, decode_responses=True)

async def init_redis():
    try:
        await redis_client.ping()
        logger.info("Redis connected successfully")
    except Exception as e:
        logger.warning(f"Redis connection failed: {e}")


"""
Redis client for pub/sub functionality
"""

import os
import json
import logging
import asyncio
from typing import Dict, Any, Optional, Callable, List
from contextlib import asynccontextmanager

import redis.asyncio as redis
from redis.asyncio import Redis

logger = logging.getLogger(__name__)


class RedisClient:
    """Redis client for pub/sub operations"""
    
    def __init__(self, host: str = None, port: int = None, db: int = 0):
        self.host = host or os.getenv("REDIS_HOST", "redis")
        self.port = port or int(os.getenv("REDIS_PORT", "6379"))
        self.db = db
        self.redis: Optional[Redis] = None
        self.pubsub: Optional[redis.client.PubSub] = None
        self._subscribers: Dict[str, List[Callable]] = {}
        self._running = False
    
    async def connect(self):
        """Connect to Redis"""
        try:
            self.redis = redis.Redis(
                host=self.host,
                port=self.port,
                db=self.db,
                decode_responses=True,
                socket_keepalive=True,
                socket_keepalive_options={},
                retry_on_timeout=True,
                health_check_interval=30
            )
            
            # Test connection
            await self.redis.ping()
            logger.info(f"Connected to Redis at {self.host}:{self.port}")
            
        except Exception as e:
            logger.error(f"Failed to connect to Redis: {e}")
            raise
    
    async def disconnect(self):
        """Disconnect from Redis"""
        if self.pubsub:
            await self.pubsub.close()
        if self.redis:
            await self.redis.close()
        self._running = False
        logger.info("Disconnected from Redis")
    
    async def publish(self, channel: str, message: Dict[str, Any]):
        """Publish message to Redis channel"""
        if not self.redis:
            raise RuntimeError("Redis client not connected")
        
        try:
            message_json = json.dumps(message)
            result = await self.redis.publish(channel, message_json)
            logger.debug(f"Published message to channel {channel}: {result} subscribers")
            return result
        except Exception as e:
            logger.error(f"Failed to publish message to channel {channel}: {e}")
            raise
    
    async def subscribe(self, channel: str, callback: Callable[[Dict[str, Any]], None]):
        """Subscribe to Redis channel"""
        if channel not in self._subscribers:
            self._subscribers[channel] = []
        
        # Проверяем, не добавлен ли уже этот callback
        if callback not in self._subscribers[channel]:
            self._subscribers[channel].append(callback)
            logger.info(f"Subscribed to channel {channel}")
        else:
            logger.debug(f"Callback already subscribed to channel {channel}")
        
        # Start listener if not running
        if not self._running:
            logger.info(f"Starting Redis listener for first subscription to channel: {channel}")
            asyncio.create_task(self.start_listening())
        # If listener is already running, subscribe to new channel
        elif self.pubsub:
            logger.info(f"Adding new channel to existing listener: {channel}")
            await self.pubsub.subscribe(channel)
        else:
            logger.info(f"Listener not ready, channel {channel} will be subscribed when listener starts")
    
    async def unsubscribe(self, channel: str, callback: Callable[[Dict[str, Any]], None] = None):
        """Unsubscribe from Redis channel"""
        if channel in self._subscribers:
            if callback:
                if callback in self._subscribers[channel]:
                    self._subscribers[channel].remove(callback)
            else:
                del self._subscribers[channel]
            logger.info(f"Unsubscribed from channel {channel}")
    
    async def start_listening(self):
        """Start listening for messages on subscribed channels"""
        if not self.redis:
            raise RuntimeError("Redis client not connected")
        
        self.pubsub = self.redis.pubsub()
        self._running = True
        logger.info(f"Redis listener started, _running={self._running}")
        
        try:
            logger.info(f"Starting Redis pubsub listener on channels: {list(self._subscribers.keys())}")
            
            # Subscribe to all channels
            if self._subscribers:
                await self.pubsub.subscribe(*self._subscribers.keys())
                logger.info(f"Subscribed to channels: {list(self._subscribers.keys())}")
            
            async for message in self.pubsub.listen():
                if message["type"] == "message":
                    channel = message["channel"]
                    data = message["data"]
                    
                    try:
                        message_data = json.loads(data)
                        logger.info(f"Processing message from channel {channel}")
                        await self._handle_message(channel, message_data)
                    except json.JSONDecodeError as e:
                        logger.error(f"Failed to decode message from channel {channel}: {e}")
                    except Exception as e:
                        logger.error(f"Error handling message from channel {channel}: {e}")
                        
        except Exception as e:
            logger.error(f"Error in Redis pubsub listener: {e}")
        finally:
            self._running = False
            logger.info("Redis listener stopped")
    
    async def _handle_message(self, channel: str, message_data: Dict[str, Any]):
        """Handle incoming message from Redis channel"""
        if channel in self._subscribers:
            for callback in self._subscribers[channel]:
                try:
                    if asyncio.iscoroutinefunction(callback):
                        await callback(message_data)
                    else:
                        callback(message_data)
                except Exception as e:
                    logger.error(f"Error in callback for channel {channel}: {e}")
    
    async def stop_listening(self):
        """Stop listening for messages"""
        self._running = False
        if self.pubsub:
            await self.pubsub.close()
    
    @property
    def is_connected(self) -> bool:
        """Check if Redis is connected"""
        return self.redis is not None
    
    @property
    def is_listening(self) -> bool:
        """Check if listening for messages"""
        return self._running


class NotificationRedisClient(RedisClient):
    """Specialized Redis client for notification service with pattern subscriptions"""
    
    def __init__(self, host: str = None, port: int = None, db: int = 0):
        super().__init__(host, port, db)
        self.instance_id = os.getenv("INSTANCE_ID", f"instance_{os.getpid()}")
    
    async def publish_notification(self, user_id: int, notification_data: Dict[str, Any]):
        """Publish notification to user-specific channel"""
        channel = f"notification:{user_id}"
        message = {
            "user_id": user_id,
            "instance_id": self.instance_id,
            "timestamp": asyncio.get_event_loop().time(),
            "data": notification_data
        }
        return await self.publish(channel, message)
    
    async def publish_broadcast(self, notification_data: Dict[str, Any]):
        """Publish broadcast notification to all instances"""
        channel = "notifications:broadcast"
        message = {
            "instance_id": self.instance_id,
            "timestamp": asyncio.get_event_loop().time(),
            "data": notification_data
        }
        return await self.publish(channel, message)
    
    async def subscribe_to_user_notifications(self, user_id: int, callback: Callable[[Dict[str, Any]], None]):
        """Subscribe to user-specific notifications"""
        channel = f"notification:{user_id}"
        await self.subscribe(channel, callback)
    
    async def subscribe_to_broadcast(self, callback: Callable[[Dict[str, Any]], None]):
        """Subscribe to broadcast notifications"""
        channel = "notifications:broadcast"
        await self.subscribe(channel, callback)
    
    async def unsubscribe_from_user_notifications(self, user_id: int, callback: Callable[[Dict[str, Any]], None] = None):
        """Unsubscribe from user-specific notifications"""
        channel = f"notification:{user_id}"
        await self.unsubscribe(channel, callback)
    
    async def unsubscribe_from_broadcast(self, callback: Callable[[Dict[str, Any]], None] = None):
        """Unsubscribe from broadcast notifications"""
        channel = "notifications:broadcast"
        await self.unsubscribe(channel, callback)

    async def subscribe_to_pattern(self, pattern: str, callback: Callable[[Dict[str, Any]], None]):
        """Subscribe to Redis pattern (e.g., notification:*)"""
        if pattern not in self._subscribers:
            self._subscribers[pattern] = []
        
        if callback not in self._subscribers[pattern]:
            self._subscribers[pattern].append(callback)
            logger.info(f"Subscribed to pattern {pattern}")
        
        # Start listener if not running
        if not self._running:
            logger.info(f"Starting Redis listener for pattern subscription: {pattern}")
            asyncio.create_task(self.start_pattern_listening())
        elif self.pubsub:
            logger.info(f"Adding pattern to existing listener: {pattern}")
            await self.pubsub.psubscribe(pattern)
        else:
            logger.info(f"Listener not ready, pattern {pattern} will be subscribed when listener starts")

    async def unsubscribe_from_pattern(self, pattern: str, callback: Callable[[Dict[str, Any]], None] = None):
        """Unsubscribe from Redis pattern"""
        if pattern in self._subscribers:
            if callback:
                if callback in self._subscribers[pattern]:
                    self._subscribers[pattern].remove(callback)
            else:
                del self._subscribers[pattern]
            logger.info(f"Unsubscribed from pattern {pattern}")

    async def start_pattern_listening(self):
        """Start listening for messages on pattern subscriptions"""
        if not self.redis:
            raise RuntimeError("Redis client not connected")
        
        self.pubsub = self.redis.pubsub()
        self._running = True
        logger.info(f"Redis pattern listener started, _running={self._running}")
        
        try:
            logger.info(f"Starting Redis pattern listener on patterns: {list(self._subscribers.keys())}")
            
            # Subscribe to all patterns
            if self._subscribers:
                await self.pubsub.psubscribe(*self._subscribers.keys())
                logger.info(f"Subscribed to patterns: {list(self._subscribers.keys())}")
            
            async for message in self.pubsub.listen():
                if message["type"] == "pmessage":
                    pattern = message["pattern"]
                    channel = message["channel"]
                    data = message["data"]
                    
                    try:
                        message_data = json.loads(data)
                        logger.info(f"Processing message from pattern {pattern}, channel {channel}")
                        await self._handle_pattern_message(pattern, channel, message_data)
                    except json.JSONDecodeError as e:
                        logger.error(f"Failed to decode message from pattern {pattern}, channel {channel}: {e}")
                    except Exception as e:
                        logger.error(f"Error handling message from pattern {pattern}, channel {channel}: {e}")
                        
        except Exception as e:
            logger.error(f"Error in Redis pattern listener: {e}")
        finally:
            self._running = False
            logger.info("Redis pattern listener stopped")

    async def _handle_pattern_message(self, pattern: str, channel: str, message_data: Dict[str, Any]):
        """Handle incoming message from Redis pattern"""
        if pattern in self._subscribers:
            for callback in self._subscribers[pattern]:
                try:
                    if asyncio.iscoroutinefunction(callback):
                        await callback(message_data)
                    else:
                        callback(message_data)
                except Exception as e:
                    logger.error(f"Error in callback for pattern {pattern}: {e}")


# Global Redis client instance
redis_client: Optional[NotificationRedisClient] = None


async def get_redis_client() -> NotificationRedisClient:
    """Get global Redis client instance"""
    global redis_client
    if redis_client is None:
        redis_client = NotificationRedisClient()
        await redis_client.connect()
    return redis_client


async def close_redis_client():
    """Close global Redis client"""
    global redis_client
    if redis_client:
        await redis_client.disconnect()
        redis_client = None 


import os
import sys
import uuid
import logging
from contextvars import ContextVar
from typing import Any, Dict

import structlog
from structlog.contextvars import bind_contextvars, clear_contextvars

# Context variables для хранения контекста запроса
trace_id_var: ContextVar[str] = ContextVar("trace_id", default="")
user_id_var: ContextVar[str] = ContextVar("user_id", default="")
instance_id_var: ContextVar[str] = ContextVar("instance_id", default="")

# Генерируем уникальный ID для экземпляра сервиса
INSTANCE_ID = str(uuid.uuid4())[:8]


def get_trace_id() -> str:
    """Получить текущий trace_id из контекста"""
    return trace_id_var.get()


def get_user_id() -> str:
    """Получить текущий user_id из контекста"""
    return user_id_var.get()


def set_trace_id(trace_id: str) -> None:
    """Установить trace_id в контекст"""
    trace_id_var.set(trace_id)


def set_user_id(user_id: str) -> None:
    """Установить user_id в контекст"""
    user_id_var.set(user_id)


def clear_context() -> None:
    """Очистить весь контекст"""
    clear_contextvars()


def configure_logging(log_level: int = logging.INFO) -> None:
    """Настроить структурированное логирование"""
    
    # Настройка базового логирования
    logging.basicConfig(
        level=log_level,
        format="%(message)s",
        stream=sys.stdout
    )
    
    # Настройка процессоров structlog
    processors = [
        # Добавляем контекстные переменные
        structlog.contextvars.merge_contextvars,
        
        # Добавляем временную метку
        structlog.processors.TimeStamper(fmt="iso"),
        
        # Добавляем уровень логирования
        structlog.stdlib.add_log_level,
        
        # Добавляем информацию о стеке
        structlog.processors.StackInfoRenderer(),
        
        # Добавляем информацию об исключениях
        structlog.processors.format_exc_info,
        
        # Добавляем service и instance_id
        _add_service_info,
        
        # Рендерим в JSON
        structlog.processors.JSONRenderer()
    ]
    
    # Конфигурируем structlog
    structlog.configure(
        processors=processors,
        wrapper_class=structlog.stdlib.BoundLogger,
        logger_factory=structlog.stdlib.LoggerFactory(),
        cache_logger_on_first_use=True,
    )


def _add_service_info(logger: Any, method_name: str, event_dict: Dict[str, Any]) -> Dict[str, Any]:
    """Добавить информацию о сервисе в каждый лог"""
    # Определяем имя сервиса из переменной окружения или по умолчанию
    service_name = os.getenv("SERVICE_NAME", "unknown_service")
    event_dict["service"] = service_name
    event_dict["instance_id"] = INSTANCE_ID
    
    # Добавляем trace_id и user_id если они есть в контексте
    trace_id = get_trace_id()
    if trace_id:
        event_dict["trace_id"] = trace_id
    
    user_id = get_user_id()
    if user_id:
        event_dict["user_id"] = user_id
    
    return event_dict


def get_logger(name: str = None) -> structlog.stdlib.BoundLogger:
    """Получить структурированный логгер"""
    if name is None:
        name = __name__
    return structlog.get_logger(name) 


"""
Shared async database configuration and repository base classes
Implements Repository pattern for clean architecture using async SQLAlchemy with asyncpg
"""

import os
import random
from abc import ABC, abstractmethod
from typing import Generic, TypeVar, List, Optional, Dict, Any, AsyncGenerator
from contextlib import asynccontextmanager

from sqlalchemy.ext.asyncio import create_async_engine, AsyncSession, async_sessionmaker
from sqlalchemy.orm import declarative_base
from sqlalchemy import select, delete, update, text, Executable, inspect
from sqlalchemy.sql.dml import Update, Delete, Insert

from dotenv import load_dotenv, find_dotenv
load_dotenv(find_dotenv())

# Type variables for generic repository
T = TypeVar('T')
ID = TypeVar('ID')

# Base class for all models
Base = declarative_base()

# 🔧 Global engines and session factory for connection pooling
_engines = None
_session_factory = None

def get_database_urls() -> (str, List[str]):
    """Get database URLs for write and read replicas from environment variables."""
    write_url = os.getenv("DATABASE_URL")
    if not write_url:
        raise ValueError("DATABASE_URL environment variable is not set.")

    read_urls_str = os.getenv("READ_REPLICA_URLS")
    read_urls = [url.strip() for url in read_urls_str.split(',')] if read_urls_str else []
    
    return write_url, read_urls

def _create_engine_internal(database_url: str, app_name_suffix: str = ""):
    """Internal function to create an async engine."""
    is_sqlite = "sqlite" in database_url.lower()
    app_name = os.getenv("APP_NAME", "unknown_service")
    if app_name_suffix:
        app_name = f"{app_name}{app_name_suffix}"

    if is_sqlite:
        return create_async_engine(
            database_url,
            pool_pre_ping=True,
            pool_recycle=300,
            echo=False
        )
    else:
        return create_async_engine(
            database_url,
            pool_size=10,
            max_overflow=20,
            pool_pre_ping=True,
            pool_recycle=300,
            echo=False,
            connect_args={
                "server_settings": {
                    "application_name": app_name
                }
            }
        )

def get_engines() -> Dict[str, Any]:
    """Get global async engines for read and write."""
    global _engines
    if _engines is None:
        write_url, read_urls = get_database_urls()
        
        write_engine = _create_engine_internal(write_url, app_name_suffix="_writer")
        
        if read_urls:
            read_engines = [_create_engine_internal(url, app_name_suffix=f"_reader_{i}") for i, url in enumerate(read_urls)]
        else:
            # If no read replicas are configured, use the write engine for reads as a fallback.
            read_engines = [write_engine]
            
        _engines = {
            'write': write_engine,
            'read': read_engines
        }
    return _engines

class RoutingAsyncSession(AsyncSession):
    """
    A custom AsyncSession that routes queries to read or write replicas.
    - Write queries (INSERT, UPDATE, DELETE) go to the 'write' engine.
    - Queries within a transaction go to the 'write' engine.
    - Read queries (SELECT) go to a randomly chosen 'read' engine.
    """
    def get_bind(self, mapper=None, clause=None, **kw):
        engines = get_engines()
        
        # If inside a transaction, always use the write database to ensure consistency.
        if self.in_transaction() or self.in_nested_transaction():
            return engines['write']

        # If the statement is a write operation, use the write database.
        if isinstance(clause, (Insert, Update, Delete)):
            return engines['write']
            
        # For SELECT statements, use a read replica.
        if isinstance(clause, (Executable)):
             # A simple heuristic to check for SELECT statements.
            if clause.is_select:
                return random.choice(engines['read'])

        # Fallback to the write engine for any other case.
        return engines['write']


def get_session_factory():
    """Get global session factory that uses the routing session."""
    global _session_factory
    if _session_factory is None:
        engines = get_engines()
        _session_factory = async_sessionmaker(
            class_=RoutingAsyncSession,
            bind=engines['write'],
            expire_on_commit=False
        )
    return _session_factory

async def dispose_engine():
    """Dispose of all global engines."""
    global _engines
    if _engines:
        await _engines['write'].dispose()
        for engine in _engines['read']:
            await engine.dispose()
        _engines = None

def get_session() -> AsyncSession:
    """Get a new session from the global pool"""
    return get_session_factory()()

@asynccontextmanager
async def get_session_context() -> AsyncGenerator[AsyncSession, None]:
    """Get async context-managed session from the global pool"""
    async with get_session_factory()() as session:
        yield session


class AsyncDatabaseConfig:
    """Async database configuration - now uses global engines"""

    def __init__(self, base=None):
        self.base = base or Base
        # We rely on the global engines, but can get one for operations like create_tables
        self.engine = get_engines()['write'] 
        self.async_session_factory = get_session_factory()

    def get_session(self) -> AsyncSession:
        """Return a new async session from the global pool"""
        return self.async_session_factory()

    @asynccontextmanager
    async def get_session_context(self) -> AsyncGenerator[AsyncSession, None]:
        """Return async context-managed session from the global pool"""
        async with self.async_session_factory() as session:
            yield session

    async def create_schema(self, schema_name: str):
        """Create schema if it does not exist"""
        async with self.engine.begin() as conn:
            await conn.execute(text(f'CREATE SCHEMA IF NOT EXISTS "{schema_name}"'))

    async def create_tables(self):
        """Create all tables (run manually from startup) on the write replica"""
        # If the base has a schema, create it.
        schema = getattr(self.base.metadata, "schema", None)
        if schema:
            await self.create_schema(schema)
        async with self.engine.begin() as conn:
            await conn.run_sync(self.base.metadata.create_all)

    async def drop_tables(self):
        """Drop all tables (for testing) from the write replica"""
        async with self.engine.begin() as conn:
            await conn.run_sync(self.base.metadata.drop_all)

    async def dispose(self):
        """Dispose of the engine - now calls global dispose"""
        await dispose_engine()


class AsyncBaseRepository(Generic[T, ID], ABC):
    """
    Async base repository implementing basic CRUD operations
    """

    def __init__(self, session: AsyncSession, model_class: type):
        self.session = session
        self.model_class = model_class

    @abstractmethod
    async def get_by_id(self, entity_id: ID) -> Optional[T]:
        pass

    @abstractmethod
    async def get_all(self, skip: int = 0, limit: int = 100) -> List[T]:
        pass

    @abstractmethod
    async def create(self, entity: T) -> T:
        pass

    @abstractmethod
    async def update(self, entity: T) -> T:
        pass

    @abstractmethod
    async def delete(self, entity_id: ID) -> bool:
        pass

    @abstractmethod
    async def exists(self, entity_id: ID) -> bool:
        pass


class AsyncSQLAlchemyRepository(AsyncBaseRepository[T, ID]):
    """
    Concrete async implementation using SQLAlchemy
    """

    async def get_by_id(self, entity_id: ID) -> Optional[T]:
        result = await self.session.get(self.model_class, entity_id)
        return result

    async def get_all(self, skip: int = 0, limit: int = 100) -> List[T]:
        stmt = select(self.model_class).offset(skip).limit(limit)
        result = await self.session.execute(stmt)
        return result.scalars().all()

    async def create(self, entity: T) -> T:
        self.session.add(entity)
        await self.session.commit()
        await self.session.refresh(entity)
        return entity

    async def update(self, entity: T) -> T:
        await self.session.merge(entity)
        await self.session.commit()
        await self.session.refresh(entity)
        return entity

    async def delete(self, entity_id: ID) -> bool:
        obj = await self.get_by_id(entity_id)
        if obj:
            await self.session.delete(obj)
            await self.session.commit()
            return True
        return False

    async def exists(self, entity_id: ID) -> bool:
        stmt = select(self.model_class).where(self.model_class.id == entity_id)
        result = await self.session.execute(stmt)
        return result.first() is not None

    async def filter_by(self, **kwargs) -> List[T]:
        stmt = select(self.model_class).filter_by(**kwargs)
        result = await self.session.execute(stmt)
        return result.scalars().all()

    async def find_by(self, criteria: Dict[str, Any]) -> List[T]:
        stmt = select(self.model_class)
        for key, value in criteria.items():
            if hasattr(self.model_class, key):
                stmt = stmt.where(getattr(self.model_class, key) == value)
        result = await self.session.execute(stmt)
        return result.scalars().all()

    async def count(self) -> int:
        """Get total count of entities"""
        stmt = select(self.model_class)
        result = await self.session.execute(stmt)
        return len(result.scalars().all())


class AsyncUnitOfWork:
    """
    Async Unit of Work for transactional consistency
    """

    def __init__(self, session: AsyncSession):
        self.session = session
        self._committed = False

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if exc_type:
            await self.rollback()
        elif not self._committed:
            await self.rollback()

    async def commit(self):
        await self.session.commit()
        self._committed = True

    async def rollback(self):
        await self.session.rollback()

    async def flush(self):
        await self.session.flush()


# 🔧 Instantiate config - now simplified
def get_db_config(base=None) -> AsyncDatabaseConfig:
    return AsyncDatabaseConfig(base)


# 📦 FastAPI dependency - now uses global session
async def get_db() -> AsyncGenerator[AsyncSession, None]:
    async with get_session_context() as session:
        yield session



"""
Password hashing utilities using bcrypt
Provides secure password hashing and verification
"""

from passlib.context import CryptContext

# Configure password context with bcrypt
pwd_context = CryptContext(schemes=["bcrypt"], deprecated="auto")


def hash_password(password: str) -> str:
    """
    Hash a password using bcrypt
    
    Args:
        password: Plain text password
        
    Returns:
        Hashed password string
    """
    return pwd_context.hash(password)


def verify_password(plain_password: str, hashed_password: str) -> bool:
    """
    Verify a password against its hash
    
    Args:
        plain_password: Plain text password to verify
        hashed_password: Hashed password from database
        
    Returns:
        True if password matches, False otherwise
    """
    return pwd_context.verify(plain_password, hashed_password) 


"""
Kafka client for event-driven communication between services
"""

import asyncio
import json
import logging
from typing import Dict, Any, Callable, Optional
from uuid import uuid4
from datetime import datetime

import aiokafka
from pydantic import BaseModel

from shared.events import BaseEvent, EventType

logger = logging.getLogger(__name__)


class KafkaClient:
    """Асинхронный Kafka клиент для публикации и подписки на события"""
    
    def __init__(self, bootstrap_servers: str, service_name: str):
        self.bootstrap_servers = bootstrap_servers
        self.service_name = service_name
        self.producer: Optional[aiokafka.AIOKafkaProducer] = None
        self.consumer: Optional[aiokafka.AIOKafkaConsumer] = None
        self.event_handlers: Dict[EventType, Callable] = {}
        self._running = False
    
    async def start_producer(self):
        """Запуск продюсера"""
        self.producer = aiokafka.AIOKafkaProducer(
            bootstrap_servers=self.bootstrap_servers,
            value_serializer=lambda v: json.dumps(v, default=str).encode('utf-8'),
            key_serializer=lambda k: k.encode('utf-8') if k else None
        )
        await self.producer.start()
        logger.info(f"Kafka producer started for service: {self.service_name}")
    
    async def start_consumer(self, topics: list[str], group_id: str):
        """Запуск консюмера"""
        self.consumer = aiokafka.AIOKafkaConsumer(
            *topics,
            bootstrap_servers=self.bootstrap_servers,
            group_id=group_id,
            value_deserializer=lambda m: json.loads(m.decode('utf-8')),
            auto_offset_reset='latest'
        )
        await self.consumer.start()
        logger.info(f"Kafka consumer started for service: {self.service_name}, topics: {topics}")
    
    async def publish_event(self, event: BaseEvent, topic: str = None):
        """Публикация события"""
        if not self.producer:
            await self.start_producer()
        
        # Определяем топик на основе типа события, если не указан
        if topic is None:
            topic = self._get_topic_by_event_type(event.event_type)
        
        # Устанавливаем сервис-источник
        event.source_service = self.service_name
        event.timestamp = datetime.utcnow()
        
        try:
            await self.producer.send(
                topic,
                value=event.dict(),
                key=str(event.user_id) if event.user_id else None
            )
            logger.info(f"Event published: {event.event_type} to topic: {topic}")
        except Exception as e:
            logger.error(f"Failed to publish event {event.event_type}: {e}")
            raise
    
    def register_handler(self, event_type: EventType, handler: Callable):
        """Регистрация обработчика событий"""
        self.event_handlers[event_type] = handler
        logger.info(f"Handler registered for event type: {event_type}")
    
    async def start_listening(self):
        """Запуск прослушивания событий"""
        if not self.consumer:
            raise ValueError("Consumer not started. Call start_consumer() first.")
        
        self._running = True
        logger.info(f"Started listening for events in service: {self.service_name}")
        
        try:
            async for message in self.consumer:
                if not self._running:
                    break
                
                try:
                    event_data = message.value
                    event_type = EventType(event_data.get('event_type'))
                    
                    if event_type in self.event_handlers:
                        logger.info(f"Processing event: {event_type}")
                        await self.event_handlers[event_type](event_data)
                    else:
                        logger.debug(f"No handler for event type: {event_type}")
                        
                except Exception as e:
                    logger.error(f"Error processing message: {e}")
                    # В production здесь должна быть логика retry/dead letter queue
        except Exception as e:
            logger.error(f"Error in event listener: {e}")
            raise
    
    async def stop(self):
        """Остановка клиента"""
        self._running = False
        
        if self.producer:
            await self.producer.stop()
            logger.info("Kafka producer stopped")
        
        if self.consumer:
            await self.consumer.stop()
            logger.info("Kafka consumer stopped")
    
    def _get_topic_by_event_type(self, event_type: EventType) -> str:
        """Определение топика по типу события"""
        if event_type.value.startswith('user.'):
            return 'user-events'
        elif event_type.value.startswith('learning.'):
            return 'learning-events'
        elif event_type.value.startswith('ai.'):
            return 'ai-events'
        elif event_type.value.startswith('notification.'):
            return 'notification-events'
        else:
            return 'general-events'


class EventPublisher:
    """Упрощенный интерфейс для публикации событий"""
    
    def __init__(self, kafka_client: KafkaClient):
        self.kafka_client = kafka_client
    
    async def publish_user_registered(self, user_id: int, user_data: Dict[str, Any]):
        """Публикация события регистрации пользователя"""
        from shared.events import UserRegisteredEvent
        
        event = UserRegisteredEvent(
            event_id=str(uuid4()),
            timestamp=datetime.utcnow(),
            user_id=user_id,
            source_service=self.kafka_client.service_name,
            data=user_data
        )
        
        await self.kafka_client.publish_event(event)
    
    async def publish_track_generation_requested(
        self, 
        user_id: int, 
        track_data: Dict[str, Any],
        correlation_id: str = None
    ):
        """Публикация события запроса генерации трека"""
        from shared.events import TrackGenerationRequestedEvent
        
        event = TrackGenerationRequestedEvent(
            event_id=str(uuid4()),
            timestamp=datetime.utcnow(),
            user_id=user_id,
            correlation_id=correlation_id or str(uuid4()),
            source_service=self.kafka_client.service_name,
            data=track_data
        )
        
        await self.kafka_client.publish_event(event)
    
    async def publish_notification_requested(
        self, 
        user_id: int, 
        notification_data: Dict[str, Any]
    ):
        """Публикация события запроса уведомления"""
        from shared.events import NotificationRequestedEvent
        
        event = NotificationRequestedEvent(
            event_id=str(uuid4()),
            timestamp=datetime.utcnow(),
            user_id=user_id,
            source_service=self.kafka_client.service_name,
            data=notification_data
        )
        
        await self.kafka_client.publish_event(event)


from .config import configure_logging, get_logger, set_trace_id, set_user_id, clear_context, get_trace_id, get_user_id
from structlog import get_logger as structlog_get_logger

configure_logging()
log = structlog_get_logger()

__all__ = [
    "configure_logging",
    "get_logger", 
    "log",
    "set_trace_id",
    "set_user_id", 
    "clear_context",
    "get_trace_id",
    "get_user_id"
] 